#include "PKW.h"
#include "FzgVerhalten.h"
#include "Weg.h"
#include "SimuClient.h"
#include <math.h>

#include <string>
#include <iostream>
#include <iomanip>
using namespace std;

#define EPSILON 0.0166 //ca. 1 Minute wenn die Zeit in Stunden gemessen wird

extern double dGlobaleZeit;

//################### Konstruktoren/Destruktor ################################
PKW::PKW(void)
{
}

PKW::PKW(string sIniName, double dVMax, double dConsumption, double dTCapacity)
	: Fahrzeug(sIniName, dVMax)
{//Konstruktor, der zusaetzlich die Anweisungen des erweiterten Fahrzeugkonstruktors aufruft.
	p_dVerbrauch = dConsumption;
	p_dTankvolumen = dTCapacity;
	p_dTankinhalt = p_dTankvolumen / 2;
}


PKW::~PKW(void)//Destruktor
{
}

//######################## Wirkfunktionen ######################################
double PKW::dTanken(double dMenge) //Die Tankfunktion
{
	if(dMenge != 0.0) //Es wurde eine Menge vorgegeben
	{
		if(p_dTankvolumen - p_dTankinhalt > dMenge)
		{//es ist mehr Platz im Tank, als Treibstoff dazukommen soll
			p_dTankinhalt += dMenge;
		}
		else
		{//es ist zu wenig Platz im Tank, um die gegebene Menge Treibstoff aufzunehmen
			dMenge = p_dTankvolumen - p_dTankinhalt;
			p_dTankinhalt = p_dTankvolumen;
		}
	}
	else //es wird vollgetankt
	{
		dMenge = p_dTankvolumen - p_dTankinhalt;
		p_dTankinhalt = p_dTankvolumen;
	}
	return dMenge;
}

double PKW::dVerbrauch(void) const //gibt den Gesamtverbrauch zurueck
{
	return p_dVerbrauch * (p_dGesamtStrecke / 100);
}

void PKW::vAbfertigung(void) //die Abfertigungsfunktion
{
	if(p_dTankinhalt > 0.0 && fabs(dGlobaleZeit - p_dZeit) > EPSILON) 
	{//PKW hat noch Treibstoff und muss in diesem Zeitschritt abgefertigt werden
		double dMerkZeit = p_dZeit; //Zeit zwischenspeichern, wird im nchsten Schritt resetet
		Fahrzeug::vAbfertigung(); //allgemeine Abfertigung
		// Tankinhalt neu ausrechnen
		p_dTankinhalt -= p_dVerbrauch * 
		(p_pVerhalten->dStrecke(this, fabs(dGlobaleZeit - dMerkZeit))/100);
		if(p_dTankinhalt <= EPSILON) p_dTankinhalt = 0.0; //ab jetzt ist der Tank leer
	}
	//damit PKW mit leerem Tank nicht nach dem Auftanken bei leerem Tank alle km nachholen.
	else p_dZeit = dGlobaleZeit;
}

void PKW::vAusgabe(void) const //die Ausgabe
{
	Fahrzeug::vAusgabe();
	cout << setiosflags(ios::fixed) << setiosflags(ios::right)
		<<setw(10) << setprecision(2) << setbase(10) << p_dTankinhalt
		<<setw(15) << setprecision(2) << setbase(10) << dVerbrauch()
		<< resetiosflags(ios::fixed) << resetiosflags(ios::right);
}

ostream& PKW::ostreamAusgabe(ostream& ostreamrefZiel)
{
	Fahrzeug::ostreamAusgabe(ostreamrefZiel) 
		<< setiosflags(ios::fixed) << setiosflags(ios::right)
		<<setw(10) << setprecision(2) << setbase(10) << p_dTankinhalt
		<<setw(15) << setprecision(2) << setbase(10) << dVerbrauch()
		<< resetiosflags(ios::fixed) << resetiosflags(ios::right);
	return ostreamrefZiel;
}

double PKW::dGeschwindigkeit(void) const
{
	if(p_pVerhalten->pGetAktuellerWeg()->dGetLimit() >= p_dMaxGeschwindigkeit)
		return p_dMaxGeschwindigkeit;
	else
		return p_pVerhalten->pGetAktuellerWeg()->dGetLimit();
}

void PKW::vZeichnen(Weg* pIniWeg)
{
	bZeichnePKW(p_sName, //Fahrzeugname
		pIniWeg->sGetName(), //Wegname
		p_dAbschnittStrecke / pIniWeg->dGetLaenge(), //Fortschritt auf dem Weg in %
		dGeschwindigkeit(),
		p_dTankinhalt);
}
//(C) MG, WiSe 2011/2012