#include "Weg.h"
#include "LazyListe.h"

#include "Fahrzeug.h"
#include "FahrAusnahme.h"
#include "Kreuzung.h"
#include <math.h>

#include <iostream>
#include <iomanip>
using namespace std;

#define EPSILON 0.0166 //ca. 1 Minute wenn die Zeit in Stunden gemessen wird.

//globale Zeit bekanntmachen
extern double dGlobaleZeit;

//########################### Konstruktoren, Destruktor ###############################
Weg::Weg(void) :AktivesVO(){} //Standard-Konstruktor

Weg::Weg(string sIniName, double dIniLaenge, Begrenzung eIniLimit, bool Ueberholverbot) //erweiterter Konstruktor
	: AktivesVO()
{
	p_sName = sIniName;
	p_dLaenge = dIniLaenge;
	p_eLimit = eIniLimit;
	p_bUeberholverbot = Ueberholverbot;
	p_dSchranke = p_dLaenge;
	p_pRueckweg = NULL;
	p_pEndKreuzung = NULL;
}

Weg::~Weg(void){}

//########################### Wirkfunktionen #############################################
ostream& Weg::ostreamAusgabe(ostream& ostreamrefZiel)
{
	AktivesVO::ostreamAusgabe(ostreamrefZiel);
	ostreamrefZiel << setiosflags(ios::right) << setiosflags(ios::fixed) 
		<< setw(10) << setprecision(2) << p_dLaenge
		<< resetiosflags(ios::fixed) << resetiosflags(ios::right);

	if(p_pFahrzeuge.empty()) 
		return ostreamrefZiel; //leere Liste muss nicht ausgegebene werden
	else
	{
		ostreamrefZiel << " ( ";
		LazyListe<Fahrzeug*>::iterator i;
		for(i = p_pFahrzeuge.begin(); i != p_pFahrzeuge.end(); ++i) 
		{
			ostreamrefZiel << (*i)->sGetName() << " "; 
		}

		ostreamrefZiel << ")";
	}
	return ostreamrefZiel;
}

void Weg::vListeAusgabe(void)
{
	if(p_pFahrzeuge.empty()) 
		return; //leere Liste muss nicht ausgegebene werden
	else
	{
		LazyListe<Fahrzeug*>::iterator i;
		for(i = p_pFahrzeuge.begin(); i != p_pFahrzeuge.end(); ++i) 
			cout << (**i) << endl; 
	}
}

void Weg::vAbfertigung()
{
	p_pFahrzeuge.vAktualisieren(); //Aufgenommene Fahrzeuge eintragen

	if(p_pFahrzeuge.empty()) return; //Ein Weg ohne Fahrzeuge muss nicht abgefertigt werden

	LazyListe<Fahrzeug*>::iterator i;
	p_dSchranke = p_dLaenge; //Schranke reset
	for(i = p_pFahrzeuge.begin(); i != p_pFahrzeuge.end(); ++i)
	{
		try 
		{ (*i)->vAbfertigung(); }
		catch(FahrAusnahme* aExc) 
		{ aExc->vBearbeiten(); }

		if(bGrafikAktiv) (*i)->vZeichnen(this);
	}
	p_pFahrzeuge.vAktualisieren();
}

void Weg::vAnnahme(Fahrzeug* pAufnehmFahrzeug) //setzt ein fahrendes Fahrzeug an den Anfang dieses Weges
{
	p_pFahrzeuge.push_back(pAufnehmFahrzeug);
	pAufnehmFahrzeug->vNeueStrecke(this); //Fahrzeug mit dem Weg bekannt machen

}

void Weg::vAnnahme(Fahrzeug* pAufnehmFahrzeug, double dIniStartZeit) //setzt ein parkendes Fahrzeug...
{
	p_pFahrzeuge.push_front(pAufnehmFahrzeug);
	pAufnehmFahrzeug->vNeueStrecke(this, dIniStartZeit);
}

void Weg::vAbgabe(Fahrzeug* pAbgebeFahrzeug)
{
	if(p_pFahrzeuge.empty()) return; //leere Liste

	LazyListe<Fahrzeug*>::iterator i;
	for(i = p_pFahrzeuge.begin(); i != p_pFahrzeuge.end(); ++i)
	{
		if((*i) == pAbgebeFahrzeug)
		{
			p_pFahrzeuge.erase(i);
			return;
		}
	}
	cout << "Fehler: Fahrzeug nicht gefunden." << endl;
	exit(1);
	return;
}
//(C) MG, WiSe 2011/2012